package collector_go

import (
	confMng "loafle.com/overflow/agent_api/config_manager"
	s "loafle.com/overflow/collector_go/scheduler"
	"log"
	"sync"
	"time"
)

var (
	instance *Collector
	once     sync.Once
)

func init() {
	AddObservers()
}

func GetInstance() *Collector {
	once.Do(func() {
		instance = &Collector{}
	})
	return instance
}

func AddObservers() {
	go handleConfigLoaded()
}

type Collector struct {
	scheduler   s.Scheduler
	cm          confMng.ConfigManager
	addSensorCh chan interface{}
	remSensorCh chan interface{}
}

func (c *Collector) start(conf confMng.ConfigManager) {
	go func() {
		c.cm = conf
		c.scheduler = s.Scheduler{}
		c.scheduler.Init()
		c.addSensorCh = make(chan interface{})
		c.remSensorCh = make(chan interface{})
		handleSensorAdded(c.addSensorCh)
		handleSensorRemoved(c.remSensorCh)

		for _, conf := range c.cm.GetSensors() {
			if err := c.addSensor(conf.Id); err != nil {
				log.Println(err)
			}
		}
	}()
}

func (c *Collector) Stop() {
	cleanObserver(c.addSensorCh, c.remSensorCh)
	c.scheduler.RemoveAllSchedule()
	c.scheduler.Stop()
}

func (c *Collector) collect(id string) {

	conf := c.cm.GetSensorById(id)
	log.Printf("COLLECT %s - [ID: %s] [Crawler : %s]", time.Now(), conf.Id, conf.Crawler.Name)

	/*
		conn, err := crm.GetInstance().GetClient(conf.Crawler.Container)
		if err != nil {
			log.Println(err)
		}
		defer conn.Close()

		dc := g.NewDataClient(conn)
		in := &g.Input{}

		in.Id = id
		in.Name = g.Crawlers(g.Crawlers_value[conf.Crawler.Name])

		out, err := dc.Get(context.Background(), in)

		if err != nil {
			log.Println(err)
		}
		log.Println(out)
	*/
}

func (c *Collector) addSensor(sensorId string) error {
	sensor := c.cm.GetSensorById(sensorId)
	return c.scheduler.NewSchedule(sensorId, sensor.Schedule.Interval, c.collect)
}

func (c *Collector) removeSensor(id string) {
	if err := c.scheduler.RemoveSchedule(id); err != nil {
		log.Println(err)
		return
	}
}
